package domaine;

import outils.Liste;

import java.io.File;
import java.io.FileNotFoundException;
import java.util.LinkedList;

import application.Alarmes;
import domaine.fichiers.*;

public class ListeDAlarmes extends LinkedList<Alarme> implements Liste<Alarme>, Operable {
    private File fichier;
    private byte version;
    private boolean modificationsApportees;
    public ListeDAlarmes() {
        super();
        modificationsApportees = false;
        fichier = null;
        version = Alarmes.FILE_VERSION;
    }
    public boolean estVide() { return isEmpty(); }
    public int nombreDElements() { return size(); }
    public void ajoute(Alarme element) { add(element); trie(); }
    public void trie() { sort((Alarme a1, Alarme a2) -> a1.compareTo(a2)); }
    public void retire(int index) { remove(index); }
    public void retire(Alarme element) { remove(element); }
    public Alarme element(int index) { return get(index); }
    public void efface() { clear(); }
    public String nomAffichable() {
        String nom = (modificationsApportees) ? "*" : "";
        if (fichier == null) nom += "Sans nom";
        else nom += fichier.getName();
        return nom;
    }
    public boolean aEteModifiee() {
        return modificationsApportees;
    }
    public void modifiee() {
        modificationsApportees = true;
    }
    public void realise(Operation op) throws Exception {
        op.opereSur(this);
    }
    public ResultatEnregistrementChargement enregistreSous(int versionDeFichier, String nomDeFichier) {
        fichier = new File(nomDeFichier);
        EnregistrementOuChargement enregistrement = null;
        try {
        	enregistrement = GestionDeFichiers.operationPourEnregistrer(versionDeFichier, nomDeFichier);
			this.realise(enregistrement);
        } catch (FileNotFoundException ec) {
        	return ResultatEnregistrementChargement.ERREUR_DE_CREATION;
        } catch (VersionDeFichierNonGeree ev) {
        	return ResultatEnregistrementChargement.ERREUR_DE_VERSION;
    	} catch (Exception e) {
    		e.printStackTrace();
    		return ResultatEnregistrementChargement.ERREUR_D_ECRITURE;
    	} finally {
    		if (enregistrement != null) enregistrement.fermeFichier();
    	}
        modificationsApportees = false;
        version = (byte)versionDeFichier;
        return ResultatEnregistrementChargement.REUSSI;
    }
    public ResultatEnregistrementChargement enregistre() { return enregistreSous(version, fichier.getAbsolutePath()); }
    public ResultatEnregistrementChargement charge(String nomDeFichier) {
        fichier = new File(nomDeFichier);
    	EnregistrementOuChargement chargement = null;
        try {
        	chargement = GestionDeFichiers.operationPourCharger(nomDeFichier);
        	clear();
        	this.realise(chargement);
        } catch (FileNotFoundException eo) {
        	return ResultatEnregistrementChargement.ERREUR_D_OUVERTURE;
        } catch (VersionDeFichierNonGeree ev) {
        	return ResultatEnregistrementChargement.ERREUR_DE_VERSION;
    	} catch (Exception e) {
    		return ResultatEnregistrementChargement.ERREUR_DE_LECTURE;
    	} finally {
    		if (chargement != null) chargement.fermeFichier();
    	}
        modificationsApportees = false;
        return ResultatEnregistrementChargement.REUSSI;
    }
}
