package domaine;

import static org.junit.jupiter.api.Assertions.*;

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;
import java.util.Iterator;

import org.junit.jupiter.api.BeforeAll;
import org.junit.jupiter.api.Test;

import domaine.fichiers.ResultatEnregistrementChargement;
import domaine.temps.MauvaisFormatDHeure;
import domaine.temps.MauvaiseValeurDesHeures;
import domaine.temps.MauvaiseValeurDesMinutes;

class TestEnregistrementEtChargementDeListeDAlarmes {
	private static ListeDAlarmes lst;
	private static ListeDAlarmes lue;
	@BeforeAll
	public static void initialiseListe() throws MauvaisFormatDHeure, MauvaiseValeurDesHeures, MauvaiseValeurDesMinutes {
        lst = new ListeDAlarmes();
        lst.ajoute(new Alarme("huit heures", 8, 0));
        lst.ajoute(new Alarme("neuf heures", 9, 0));
        lst.ajoute(new Alarme("douze heures trente", 12, 30));
        lst.ajoute(new Alarme("treize heures trente", 13, 30));
        lst.ajoute(new Alarme("dix-sept heures", 17, 0));
        lst.ajoute(new Alarme("vingt heures", 20, 0));
        lue = new ListeDAlarmes();
	}
    @Test
    public void test01EnregistrementEtChargementTexteV1() throws MauvaisFormatDHeure, MauvaiseValeurDesHeures, MauvaiseValeurDesMinutes {
    	assertEquals(ResultatEnregistrementChargement.REUSSI, lst.enregistreSous(1, "testV1.txt"));
    	assertEquals(ResultatEnregistrementChargement.REUSSI, lue.charge("testV1.txt"));
        assertEquals(lue.nombreDElements(), lst.nombreDElements());
        Iterator<Alarme> itlst = lst.iterator();
        Iterator<Alarme> itlue = lue.iterator();
        Alarme alLst, alLue;
        while(itlst.hasNext()) {
        	alLst = itlst.next();
        	alLue = itlue.next();
            assertEquals(0, alLst.heure().compareTo(alLue.heure()));
            assertEquals("sans nom", alLue.nom());
        }
    }
    @Test
    public void test02EnregistrementEtChargementTexteV2() throws MauvaisFormatDHeure, MauvaiseValeurDesHeures, MauvaiseValeurDesMinutes {
    	assertEquals(ResultatEnregistrementChargement.REUSSI, lst.enregistreSous(2, "testV2.txt"));
    	assertEquals(ResultatEnregistrementChargement.REUSSI, lue.charge("testV2.txt"));
        assertEquals(lue.nombreDElements(), lst.nombreDElements());
        Iterator<Alarme> itlst = lst.iterator();
        Iterator<Alarme> itlue = lue.iterator();
        Alarme alst, alue;
        while(itlst.hasNext()) {
        	alst = itlst.next();
        	alue = itlue.next();
            assertEquals(0, alst.compareTo(alue));
        }
    }
    @Test
    public void test03EnregistrementEtChargementBinaireV1() throws MauvaisFormatDHeure, MauvaiseValeurDesHeures, MauvaiseValeurDesMinutes {
    	assertEquals(ResultatEnregistrementChargement.REUSSI, lst.enregistreSous(1, "testV1.alrm"));
    	assertEquals(ResultatEnregistrementChargement.REUSSI, lue.charge("testV1.alrm"));
        assertEquals(lue.nombreDElements(), lst.nombreDElements());
        Iterator<Alarme> itlst = lst.iterator();
        Iterator<Alarme> itlue = lue.iterator();
        Alarme alLst, alLue;
        while(itlst.hasNext()) {
        	alLst = itlst.next();
        	alLue = itlue.next();
            assertEquals(0, alLst.heure().compareTo(alLue.heure()));
            assertEquals("sans nom", alLue.nom());
        }
    }
    @Test
    public void test04EnregistrementEtChargementBinaireV2() throws MauvaisFormatDHeure, MauvaiseValeurDesHeures, MauvaiseValeurDesMinutes {
    	assertEquals(ResultatEnregistrementChargement.REUSSI, lst.enregistreSous(2, "testV2.alrm"));
    	assertEquals(ResultatEnregistrementChargement.REUSSI, lue.charge("testV2.alrm"));
        assertEquals(lue.nombreDElements(), lst.nombreDElements());
        Iterator<Alarme> itlst = lst.iterator();
        Iterator<Alarme> itlue = lue.iterator();
        while(itlst.hasNext()) {
            assertEquals(0, itlst.next().compareTo(itlue.next()));
        }
    }
    @Test
    public void test05EnregistrementXMLV1() throws MauvaisFormatDHeure, MauvaiseValeurDesHeures, MauvaiseValeurDesMinutes, IOException {
        ListeDAlarmes lst;
        lst = new ListeDAlarmes();
        lst.ajoute(new Alarme("neuf heures", 9, 0));
        lst.ajoute(new Alarme("quinze heures trente", 15, 30));
        assertEquals(ResultatEnregistrementChargement.REUSSI, lst.enregistreSous(1, "testV1.xml"));
    	BufferedReader fichier = new BufferedReader(new FileReader("testV1.xml"));
		assertEquals("<alarmes version=\"1\">", fichier.readLine());
		assertEquals("\t<alarme>", fichier.readLine());
		assertEquals("\t\t<heure>", fichier.readLine());
		assertEquals("\t\t\t09:00", fichier.readLine());
		assertEquals("\t\t</heure>", fichier.readLine());
		assertEquals("\t</alarme>", fichier.readLine());
		assertEquals("\t<alarme>", fichier.readLine());
		assertEquals("\t\t<heure>", fichier.readLine());
		assertEquals("\t\t\t15:30", fichier.readLine());
		assertEquals("\t\t</heure>", fichier.readLine());
		assertEquals("\t</alarme>", fichier.readLine());
		assertEquals("</alarmes>", fichier.readLine());
		fichier.close();
    }
    @Test
    public void test06EnregistrementXMLV2() throws MauvaisFormatDHeure, MauvaiseValeurDesHeures, MauvaiseValeurDesMinutes, IOException {
        ListeDAlarmes lst;
        lst = new ListeDAlarmes();
        lst.ajoute(new Alarme("neuf heures", 9, 0));
        lst.ajoute(new Alarme("quinze heures trente", 15, 30));
        assertEquals(ResultatEnregistrementChargement.REUSSI, lst.enregistreSous(2, "testV2.xml"));
    	BufferedReader fichier = new BufferedReader(new FileReader("testV2.xml"));
		assertEquals("<alarmes version=\"2\">", fichier.readLine());
		assertEquals("\t<alarme>", fichier.readLine());
		assertEquals("\t\t<nom>", fichier.readLine());
		assertEquals("\t\t\tneuf heures", fichier.readLine());
		assertEquals("\t\t</nom>", fichier.readLine());
		assertEquals("\t\t<heure>", fichier.readLine());
		assertEquals("\t\t\t09:00", fichier.readLine());
		assertEquals("\t\t</heure>", fichier.readLine());
		assertEquals("\t</alarme>", fichier.readLine());
		assertEquals("\t<alarme>", fichier.readLine());
		assertEquals("\t\t<nom>", fichier.readLine());
		assertEquals("\t\t\tquinze heures trente", fichier.readLine());
		assertEquals("\t\t</nom>", fichier.readLine());
		assertEquals("\t\t<heure>", fichier.readLine());
		assertEquals("\t\t\t15:30", fichier.readLine());
		assertEquals("\t\t</heure>", fichier.readLine());
		assertEquals("\t</alarme>", fichier.readLine());
		assertEquals("</alarmes>", fichier.readLine());
		fichier.close();
    }
}
